library own_dropdown;

import 'package:flutter/material.dart';

class OwnDropdown extends StatefulWidget {
  final List<String> items;
  final String title;
  final BoxDecoration decoration;
  final int defaultItemIndex;
  Function(int)? onChanged;
  final TextStyle? itemsStyle;
  final EdgeInsets? padding;
  final double space;
  final double blur;

  OwnDropdown({
    required this.items,
    required this.decoration,
    required this.title,
    this.defaultItemIndex = 0,
    this.onChanged,
    this.padding,
    this.space = 5,
    this.blur = 0.05,
    this.itemsStyle,
  });

  @override
  State<OwnDropdown> createState() => _OwnDropdownState();
}

class _OwnDropdownState extends State<OwnDropdown> {
  String mainText = "";
  OverlayEntry? _baseOverlayEntry;

  @override
  void initState() {
    mainText = widget.title;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        showDropDown();
      },
      child: Container(
        padding: widget.padding ?? EdgeInsets.all(10),
        decoration: widget.decoration,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Text(
              mainText,
              style: widget.itemsStyle,
            ),
            Icon(
              Icons.expand_more,
              color: (widget.itemsStyle?.color) ?? Colors.black,
              size: 2 * ((widget.itemsStyle?.fontSize) ?? 10),
            )
          ],
        ),
      ),
    );
  }

  bool isOpen = false;

  void showDropDown() {
    if (!isOpen) {
      _baseOverlayEntry = _overlayEntry();
      Overlay.of(context)?.insert(_baseOverlayEntry!);
      isOpen = true;
    } else {
      if (_baseOverlayEntry != null) {
        _baseOverlayEntry!.remove();
        isOpen = false;
      }
    }
  }

  OverlayEntry _overlayEntry() {
    RenderBox renderBox = context.findRenderObject()! as RenderBox;
    var size = renderBox.size;
    var offset = renderBox.localToGlobal(Offset.zero);

    return OverlayEntry(
      builder: (context) => Positioned(
        left: offset.dx,
        top: offset.dy + size.height + widget.space,
        width: size.width,
        height: MediaQuery.of(context).size.height -
            offset.dy -
            size.height -
            widget.space,
        child: Material(
          color: Colors.transparent,
          elevation: 1,
          child: Container(
            // decoration: widget.decoration,
            color: widget.decoration.color?.withOpacity(
              widget.blur,
            ),
            child: Column(
              children: [
                Container(
                  decoration: widget.decoration,
                  child: ListView.builder(
                    padding: EdgeInsets.zero,
                    itemCount: widget.items.length,
                    shrinkWrap: true,
                    itemBuilder: (context, index) {
                      return InkWell(
                        onTap: () {
                          mainText = widget.items[index];
                          widget.onChanged?.call(index);
                          showDropDown();
                          setState(() {});
                        },
                        child: Padding(
                          padding: widget.padding ?? EdgeInsets.all(10),
                          child: Text(
                            widget.items[index],
                            style: widget.itemsStyle,
                          ),
                        ),
                      );
                    },
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
